/*
 * Dynamic Particle System Library (DPS)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _DPS_H_
#define _DPS_H_

/*
 * Este eh o modulo principal do API. Para trabalhar com o API basta
 * importar este header. Toda a restante funcionalidade eh automaticamente
 * importada por este header file.
 *
 * Aqui estao definidas todas as funcoes de alto nivel que o programador
 * precisa de conhecer para trabalhar com o API.
 *
 * A utilizacao deste API baseia-se em tres fases distintas de operacao:
 *
 *      1. Definir o mundo das particulas.
 *      
 *      Basicamente temos que dizer ao DPS quais os conjuntos de particulas,
 *      forcas e objectos com que queremos trabalhar.
 *
 *      Para esse efeito construimos instrucoes semelhantes a's utilizadas pelo
 *      OpenGL para criacao de display lists.
 *
 *      Para a criacao de um Grupo de Partilas temos portanto as seguintes
 *      instrucoes:
 *
 *          1.1. DPShandle dpsGenParticleSystem(DPSsizei range);
 *                    
 *          Cria 'range' grupos de particulas e retorna um handle para 
 *          o primeiro. Os seguintes devem ser invocados por  'range + n'.
 *
 *
 *          1.2. void dpsNewParticleSystem(DPShandle handle);
 *
 *          Inicia o estado de introducao de particulas num dado grupo
 *          especificado por handle. Diferentes grupos teem que ter diferentes
 *          blocos de introducao.
 *          Podemos utilizar os seguintes modificadores de estado:
 *
 *             void dpsColor4f(DPSfloat r, DPSfloat g, DPSfloat b, DPSfloat a);
 *             void dpsColor3f(DPSfloat r, DPSfloat g, DPSfloat b);
 *             void dpsPosition3f(DPSfloat rx, DPSfloat ry, DPSfloat rz);
 *             void dpsVelocity3f(DPSfloat vx, DPSfloat vy, DPSfloat vz);
 *             void dpsMeshFunc(DPSvoid (*mesh)(int));
 *             void dpsAgef(DPSfloat age);
 *             void dpsMassf(DPSfloat mass);
 *
 *
 *           1.3. void dpsEndParticleSystem();
 *
 *           Termina o estado de introducao de particulas.
 *           Deixamos de estar a trabalhar com um grupo de particulas
 *           Especifico.
 *
 *
 *           1.4. void dpsNewParticle();      
 *
 *           Adiciona uma particula (com as propriedades de estado actuais) 
 *           ao grupo de particulas.
 *
 *
 *
 *      2. Definir o conjunto das forcas que interveem no mundo.
 *      
 *      Guerra isto eh contigo...
 *
 *
 *      3. Definir o conjunto dos objectos que vao existir no nosso mundo.
 *      
 *      Os objectos podem ser:
 *
 *            3.1.1. Objectos invisiveis com os quais uma particula choca
 *                   elasticamente ou eh repelida.
 *      
 *            3.1.2. Geradores de particulas, ou seja, pequenos objectos
 *                   invisiveis e configuraveis que geram particulas com
 *                   um dada frequencia (particulas/unidade de tempo).
 *          
 *            3.1.3. Destruidores de particulas. Sao objectos que eliminam
 *                   particulas, quer seja por elas chocarem contra eles
 *                   ou por desrespeitarem uma determinada regra estrutural,
 *                   como por exemplo serem velhas demais.
 *                   
 *                   
 *                   
 *      Gabi isto eh contigo...
 *
 */




#include "particle.h"
#include "force.h"
#include "object.h"
#include "types.h"
#include "utility.h"


/*
 **************************************************************************
 *                        CUT HERE (MIGUEL)                               *
 * Escrever apenas dentro deste bloco para nao se misturar os codigos     *
 **************************************************************************
 *
 *  Particle Systems 
 */



/*
 *  State functions
 */

void dpsParticlePosition3fv(DPSvector3f position, DPSparticle *p);
void dpsParticleColor4fv(DPSvector4f color, DPSparticle *p);
void dpsParticleVelocity3fv(DPSvector3f velocity, DPSparticle *p);


void dpsTimeIncrementf(DPSfloat dt);  // define o dt;
void dpsColor4f(DPSfloat r, DPSfloat g, DPSfloat b, DPSfloat a);
void dpsColor3f(DPSfloat r, DPSfloat g, DPSfloat b);
void dpsRGBADev(DPSfloat rdev, DPSfloat gdev, DPSfloat bdev, DPSfloat adev);
void dpsPosition3f(DPSfloat rx, DPSfloat ry, DPSfloat rz);
void dpsVelocity3f(DPSfloat vx, DPSfloat vy, DPSfloat vz);
void dpsMeshFunc(DPSvoid (*mesh)(DPSulong));
void dpsAgef(DPSfloat age);
void dpsMassf(DPSfloat mass);
void dpsMeshRoll(DPSboolean roll);
void dpsDieFunc(DPSvoid (*die)(DPShandle, DPSshandle, 
                      DPSshandle, DPSshandle));
void dpsCollideFunc(DPSvoid (*collide)(DPShandle, DPSshandle, 
                      DPSshandle, DPSshandle));




/*
 *  Particle Groups Interfaces
 */

DPSshandle    dpsGenParticleSystem(DPSsizei range);
void          dpsNewParticleSystem(DPSshandle handle);
void          dpsEndParticleSystem();
void          dpsRemoveParticle(DPSshandle shandle, DPShandle phandle);
DPShandle     dpsNewParticle();      
void          dpsDrawParticleSystem(DPSbitfield mask, DPSshandle handle);
DPSulong      dpsLiveParticles(DPSshandle handle);
void          dpsIterateParticles(DPSshandle particles, 
		                  DPSshandle forces, 
			          DPSshandle objects);


/*
 **************************************************************************
 *                        CUT HERE END (MIGUEL)                           *
 **************************************************************************
 */



// RUI ESTA EH A TUA PARTE. TENS QUE FAZER ESTAS FUNCOES...

/*
 **************************************************************************
 *                        CUT HERE (GUERRA)                               *
 * Escrever apenas dentro deste bloco para nao se misturar os codigos     *
 **************************************************************************
 *
 *  Force Systems 
 */

/*
 * Force Groups Interfaces
 */



void       dpsRemoveForce(DPSshandle shandle, DPShandle fhandle);
DPSshandle dpsGenForceSystem(DPSsizei range);
void       dpsNewForceSystem(DPSshandle handle);
void       dpsEndForceSystem();




DPShandle dpsNewExplosion(DPSfloat x, DPSfloat y, DPSfloat z,
                     DPSfloat velocity, 
   		     DPSfloat magnitude, 
		     DPSfloat stdev, 
		     DPSfloat epsilon,
		     DPSfloat age);


DPShandle dpsNewFollow(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);

DPShandle dpsNewGravity(DPSfloat x, DPSfloat y, DPSfloat z);      

DPShandle dpsNewGravitate(DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);

DPShandle dpsNewJet(DPSfloat cx, DPSfloat cy, DPSfloat cz,
   	            DPSfloat ax, DPSfloat ay, DPSfloat az,		     
	            DPSfloat magnitude, 
	            DPSfloat epsilon,
	            DPSfloat max_radius);
			
DPShandle dpsNewSpeedlimit(DPSfloat min, DPSfloat max);

DPShandle dpsNewTargetColor(DPSfloat c1, DPSfloat c2, DPSfloat c3, DPSfloat alpha, DPSfloat scale);

DPShandle dpsNewDamping(DPSfloat d0, DPSfloat d1, DPSfloat d2, DPSfloat vlowSqr, DPSfloat vhighSqr);

DPShandle dpsNewVortex(DPSfloat p0, 
    		  DPSfloat p1,  
		  DPSfloat p2, 
		  DPSfloat axis0, 
		  DPSfloat axis1,
		  DPSfloat axis2, 
		  DPSfloat magnitude, DPSfloat epsilon, DPSfloat max_radius);


DPShandle dpsNewKillOld(DPSfloat age_limit, DPSboolean  kill_less_than);


/*
 **************************************************************************
 *                        CUT HERE END (GUERRA)                           *
 **************************************************************************
 */



/*
 **************************************************************************
 *                        CUT HERE (GABI)                                 *
 * Escrever apenas dentro deste bloco para nao se misturar os codigos     *
 **************************************************************************
 *
 *  Object Systems 
 */


/*
 * Object Groups Interfaces
 */

void       dpsRemoveObject(DPSshandle shandle, DPShandle ohandle);
DPSshandle dpsGenObjectSystem(DPSsizei range);
void       dpsNewObjectSystem(DPSshandle handle);
void       dpsEndObjectSystem();


DPShandle   dpsNewSphere(DPSfloat x, DPSfloat y, DPSfloat z,
	                 DPSfloat ri, DPSfloat ro, DPSenum action, 
	   	         DPSfloat var1, DPSfloat var2, DPSfloat var3);

			 
DPShandle dpsNewPlane(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		      DPSfloat x1, DPSfloat y1, DPSfloat z1,
		      DPSenum action, DPSfloat var1, DPSfloat var2, 
		      DPSfloat var3);


DPShandle dpsNewRectangle(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSfloat x2, DPSfloat y2, DPSfloat z2,
		     DPSenum action, DPSfloat var1, 
		     DPSfloat var2, DPSfloat var3);

DPShandle dpsNewTriangle(DPSfloat x0, DPSfloat y0, DPSfloat z0,
		     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSfloat x2, DPSfloat y2, DPSfloat z2,
		     DPSenum action, DPSfloat var1, 
		     DPSfloat var2, DPSfloat var3);

DPShandle dpsNewDisc(DPSfloat x0, DPSfloat y0, DPSfloat z0,
          	     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSfloat ri, DPSfloat ro,
		     DPSenum action, DPSfloat var1, 
		     DPSfloat var2, DPSfloat var3);


DPShandle dpsNewCube(DPSfloat x0, DPSfloat y0, DPSfloat z0,
          	     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSenum action, DPSfloat var1, 
		     DPSfloat var2, DPSfloat var3);

DPShandle dpsNewCone(DPSfloat x0, DPSfloat y0, DPSfloat z0,
          	     DPSfloat x1, DPSfloat y1, DPSfloat z1,
		     DPSfloat ri, DPSfloat ro, DPSenum action,
		     DPSfloat var1, DPSfloat var2, DPSfloat var3);

DPShandle dpsNewCylinder(DPSfloat x0, DPSfloat y0, DPSfloat z0,
          	         DPSfloat x1, DPSfloat y1, DPSfloat z1,
		         DPSfloat ri, DPSfloat ro, DPSenum action,
			 DPSfloat var1, DPSfloat var2, DPSfloat var3);



/*
 **************************************************************************
 *                        CUT HERE END (GABI)                             *
 **************************************************************************
 */

#endif


