/*
 * Dynamic Particle System Library (DPS)
 * Version:  1.0
 *
 * Universidade do Minho 2000/2001
 * Licenciatura em Engenharia de Sistemas e Informatica - LESI
 *
 * Jose Miguel Araujo Ferreira (jmaferreira@hotmail.com)
 * Pedro Gabriel Ferreira      (pgabiferreira@portugalmail.pt)
 * Rui Manuel A. R. S. Guerra  (ruiguerra@hotmail.com)
 *
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _PARTICLE_H_
#define _PARTICLE_H_

/*
 * Basicamente uma particula eh definida pela sua posicao, velocidade e massa. 
 * Estas tres unidades fisicas sao suficientes para modelar um sistema
 * newtoniano de massas, sujeitas a um numero determinado de forcas.
 *
 * Na nossa abordagem ao paradigma dos sistemas de particulas utilizamos
 * tres entidades distintas de interaccao mutua:
 *
 *       1. Grupos de Particulas
 *
 *       Basicamente sao conjuntos de particulas independentes, cada uma com o 
 *       seu conjunto de atributos (r, p, m).
 *
 *
 *       2. Grupos de Forcas
 *
 *       Sao definidos varios conjuntos de forcas, que sao aplicados a um 
 *       determinado conjunto de Particulas.
 *
 *
 *       3. Grupos de Objectos
 *
 *       Definimos tambem diversos tipos de dominios restritivos a aplicar 
 *       ao movimento das particulas, como por exemplo, locais onde as 
 *       particulas estao proibidas de entrar, ou objectos invisiveis nos 
 *       quais as particulas saltam apos uma colisao.
 *       Aqui definem-se tambem objectos de geracao e destruicao de particulas. 
 *       Resumindo, temos objectos geradores, destruidores de repulsao de 
 *       particulas.
 *
 *
 * A cada iteracao do sistema, sao aplicados um conjunto de Forcas e um
 * conjunto de Restricoes a um conjunto de Particulas. Esta accao ira 
 * recalcular novas posicoes para as particulas, baseando o seu movimento
 * num dado valor de delta tempo (dt).
 * Uma vez calculadas as novas posicoes das particulas soh eh preciso 
 * desenhar o conjunto das particulas.
 *
 * Algoritomo generico de funcionamento:
 *
 *    1. dpsIterate(P, F, R, dt).
 *    2. dpsDraw(P).
 *
 */

#include <stdio.h>
#include "types.h"
#include "utility.h"
 
typedef struct _DPSparticle_ {
  DPSvector3f		position;       // position vector - r
  DPSvector3f		velocity;       // velocity vector - v
  DPSfloat 		mass; 		// particle's mass - m

  DPSfloat 		age; 		// particle's age
  DPSvector4f		color;  	// Red Green Blue Alpha

  DPSboolean            roll;              // Rotate mesh with v orientation
  DPSvoid               (*mesh)(DPSulong); // pointer to mesh function
  DPSulong              mesh_arg;          // argument o mesh function, 
                                           // basically, an iteration counter
                                        
  DPSvoid               (*collide)(DPShandle, // pointer to collide function  
                           DPSshandle,        // func(particle, particles, forces, objects)
                           DPSshandle, DPSshandle); 
  DPSvoid               (*die)(DPShandle, // pointer to collide function  
                           DPSshandle,    // func(particle, particles, forces, objects)
                           DPSshandle, DPSshandle); 
} DPSparticle; 			         // DPS particle
   

/*
 *  Function Signatures
 */

/*
 *  Memory allocation 
 */
DPSparticle* dpsNewParticle10v(
  DPSvector3f position, 
  DPSvector3f velocity,
  DPSfloat mass, 
  DPSfloat age, 
  DPSvector4f color,
  DPSvector4f colordev,
  DPSboolean roll,  
  DPSvoid (*mesh)(DPSulong), 
  DPSvoid (*die)(DPShandle, DPSshandle, DPSshandle, DPSshandle),
  DPSvoid (*collide)(DPShandle, DPSshandle, DPSshandle, DPSshandle)
);


#endif
